
VERSION = 1.21

--[[### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####
]]

-- @author Michael Martin with much appreciated code by Thomas Loockx, Roeland,
--                          gmillas, Jason Grimes, Mark Bassett & Matej Moderc.
--
-- @shortcut alt + R
--
-------------------------------------------------------------------------------
--
--                 Ol' Ready Cam (ORC) Key Frame Animator
--
-- Summary:
--    1. Move Octane's camera using camera positioning data from Blender
--       or other 3d packages exported in the proper format.
--    2. Create postioning data within Octane using shortcut keys in order
--       to perform simple key framing. Any two sequential positions within
--       an Orc file can be key framed with up to 2000 interpolations.
--
-- This script is used for positioning Octane's camera using data from an
-- Orc file allowing for camera animations or simply positioning the camera
-- using an outside program. The single frame approach allows you to work
-- interactively between Octane Standalone and Blender -- two mouseclicks or
-- two keystrokes does the trick. Move the camera,click,click...repeat.
--
-- Though written especially with Blender in mind, any 3D package can take
-- advantage of this script by exporting their own Orc files in the proper
-- format. These are simply text files that use the extension .orc for easy
-- identification. Each line of position data is considered as a frame.
-- The Blender exporter can handle key framed camera ranges or single and
-- appended camera positions. Exporting (from Blender) and importing
-- (from Octane) can be by shortcut keys or mouseclicks.
--
-- This script borrows some functionality from the script 'Load Animation Path'
-- by gmillas retaining the ability to use the FFmpeg program for generating
-- videos using the still images that are produced. The format of gmillas'
-- camera files (2 vector with files ending in .txt) is supported, but the
-- files must be renamed with the .orc extension in order to be recognized.
--
-- So 2 and 3 vector camera files are supported. An Orc file consists of
-- 6 or 9 fields separated by comas with the layout as follows:
--   1. Target vector   in x,y,z order
--   2. Position vector in x,y,z order
--   3. Up vector       in x,y,z order
-- 
-- The target and position vectors are required while the Up vector is optional.
-- The file from Blender uses all 3. The script determines whether to use 2 or 3
-- vectors based on the 1st line of the file.
--
--[[                 Here's an example of a 2 line 3 vector file:

6.155272, 1.611877, 6.939515, 6.882179, 1.712987, 7.618765, -0.073398, 0.994875, -0.069545
5.875163, 1.938877, 6.853199, 6.602699, 2.070560, 7.526513, -0.096169, 0.991292, -0.089959

]]
--
--         I lift up my eyes to the hills. From whence does my help come?
--             My help comes from the LORD, Who made heaven and earth.
--                                                       Psalm 121:1-2

------------------------------------------------------------------------------
-- Load ancillary files
------------------------------------------------------------------------------
require "Orc_bin/gui"
require "Orc_bin/dialog"
require "Orc_bin/render"
require "Orc_bin/scene"

------------------------------------------------------------------------------
-- Window that holds all assembled components
------------------------------------------------------------------------------
OrcWindow = octane.gui.create
{ 
    type     = octane.gui.componentType.WINDOW,
    text     = string.format("Ol' Ready Cam (ORC) Key Frame Animator %s", VERSION),
    children = { layout.group },
    width    = layout.group:getProperties().width,
    height   = layout.group:getProperties().height
}

Window_is_showing = false

--------------------------------------------------------------------------------
--////////////////////////////////////////////////////////////////////////////--
--------------------------------------------------------------------------------
if( scene:initialize()) then
    Window_is_showing = true
    OrcWindow:showWindow()
    Window_is_showing = false
    Render.is_dead = true
end
