
-- :::::::::::::::::::::::::::::
-- ::                         ::
-- ::       Animate Tab       ::
-- ::                         ::
-- :::::::::::::::::::::::::::::................................................

animate = widget:create()

animate:new('button', 'path') { "Output...", tooltip="Choose output folder where image files are to be written." }
                      :append('editor') { enable=true }
animate.path.path = ""  -- Save just the path portion of editor.text for later reconstruction of output names

animate.path.verify = false  -- Need to verify path if manually entered.

animate:new('check', 'forceSequence') { text="Force sequential numbering when using Step", width=260, x=25, checked=true}
animate:new('check', 'runFFmpeg')     { text="Run FFmpeg after rendering", width=260, x=25}

-------------------------------------------
animate:new('combo', 'imageFormat')
    { label={"Image Format", width=110},
      items = {
          "PNG 8-bpc",
          "PNG 16-bpc",
          "EXR untonemapped",
          "EXR tonemapped"
      }
    }
 
-------------------------------------------
animate:new('combo', 'videoFormat')
    { label={"Video Format", width=110},
      items = {
          "avi",
          "mkv",
          "mp4"
      }
   }

-------------------------------------------------------------------------------
--  Use get() for  retrieving currently selected image and file formats and
--  their extensions. When adding new formats, keep the local extension and
--  format elements in the corresponding order of animate.imageFormat and
--  animate.videoFormat.
-------------------------------------------------------------------------------
function animate:get(argv)

    local extension = { ".png", ".png", ".exr", ".exr" }
    local format    = { octane.imageSaveType.PNG8,
                        octane.imageSaveType.PNG16,
                        octane.imageSaveType.EXR,
                        octane.imageSaveType.EXRTONEMAPPED
                      }

    if argv == 'videoExtension' then
        local index = self.videoFormat.combo.selectedIx
        return "." .. self.videoFormat.combo.items[index]

    elseif argv == 'imageExtension' then
        local index = self.imageFormat.combo.selectedIx
        return extension[index]

    elseif argv == 'imageFormat' then
        local index = self.imageFormat.combo.selectedIx
        return format[index]
    end
end

--------------------------------------------------------------------------------

animate:new('slider', 'samples')       {{"Samples/px"},   max=500, log=true}
animate:new('slider', 'frameRate')     {{"Frame Rate"},   min=10, max=120, value=settings.frameRate.numeric.value }
animate:new('slider', 'shutterTime')   {{"Shutter Time"}, min=0,  max=2, step=0.001, value=settings.shutterTime.numeric.value, log=true}
animate:new('numeric','startSequence') {{"Starting #", width=60}, width=80,value=1, min=1, max=500000, step=1}

animate:new('slider','startFrame') {{"Start",width=40} }
animate:new('slider','endFrame')   {{"End",  width=40} }
animate:new('slider','step')       {{"Step", width=40}, max=100 }
animate:new('button','render')     {"Render"}
animate:new('button','stop')       {"Stop"}
animate:new('button','ffmpeg')     {"FFmpeg", tooltip="Create video from existing files whose path "..
                                     "name and prefix is shown in the Output path above."} 

--------------------------------------------------------------------------------------
-- Screen groups and layout for the Animate tab
--------------------------------------------------------------------------------------
do
    local children =
    {
        { animate.path.button, animate.path.editor }
    }
    local g1 = animate:new('group') { children, padding={10} }

    ---------------------------------------------------------------------
    children =
    {
        { animate.imageFormat.label, animate.imageFormat.combo, animate.forceSequence.check },
        { animate.videoFormat.label, animate.videoFormat.combo, animate.runFFmpeg.check }
    }
    local g2 = animate:new('group') { children, padding={2}, center=true }

    ---------------------------------------------------------------------
    children =
    {
        { animate.samples.label,     animate.samples.slider },
        { animate.frameRate.label,   animate.frameRate.slider },
        { animate.shutterTime.label, animate.shutterTime.slider },
    }
    local g3 = animate:new('group') { children, padding={1}, inset={10} }

    ---------------------------------------------------------------------
    children =
    {
        { animate.startSequence.label, animate.startSequence.numeric },
        { animate.startFrame.label,    animate.startFrame.slider },
        { animate.endFrame.label,      animate.endFrame.slider },
        { animate.step.label,          animate.step.slider }
    }
    local g4 = animate:new('group') { children, text="Frame Range",
                                    border=true, padding={2}, inset={10,10} }
    ---------------------------------------------------------------------
    children =
    {
        {  animate.render.button, animate.stop.button, animate.ffmpeg.button }
    }
    local g5 = animate:new('group') { children, padding={5,10} }

    ---------------------------------------------------------------------
    children =
    {
        { g1.group },
        { g2.group },
        { g3.group },
        { g4.group },
        { g5.group }
    }
    animate:append('group') { children, center=true, padding={1}, x=22}
end ----------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Animate functions ///////////////////////////////////////////////////////////
--------------------------------------------------------------------------------
function animate:enable(...)

    local argv = {...} if (#argv > 0) then argv = argv[1] end
 
    local enabler = cameraFile.editor.text ~= ""

    if argv.pathOn ~= nil then
        animate.path.button:updateProperties       { enable=argv.pathOn }
        animate.path.editor:updateProperties       { enable=argv.pathOn }
    else
        animate.path.button:updateProperties       { enable=enabler }
        animate.path.editor:updateProperties       { enable=enabler }
    end

    animate.startFrame.slider:updateProperties     { enable=enabler }
    animate.endFrame.slider:updateProperties       { enable=enabler }
    animate.step.slider:updateProperties           { enable=enabler }
    animate.imageFormat.combo:updateProperties     { enable=true }
    animate.videoFormat.combo:updateProperties     { enable=true }
    animate.forceSequence.check:updateProperties   { enable=true }
    animate.startSequence.numeric:updateProperties { enable=true }
    animate.runFFmpeg.check:updateProperties       { enable=true }
    animate.samples.slider:updateProperties        { enable=true }
    animate.frameRate.slider:updateProperties      { enable=true }
    animate.shutterTime.slider:updateProperties    { enable=true }
    animate.stop.button:updateProperties           { enable=false }

    if cameraFile.editor.text == "" or
        animate.path.editor.text == ""  then
        animate.render.button:updateProperties { enable=false }
    else
        animate.render.button:updateProperties { enable=true }
    end

    if animate.path.editor.text == ""  then
        animate.ffmpeg.button:updateProperties { enable=false }
    else
        animate.ffmpeg.button:updateProperties { enable=true }
    end

end

--------------------------------------------------------------------------------
function animate:disable(...)

    local argv = {...} if (#argv > 0) then argv = argv[1] end

    if argv.pathOn ~= nil then
        animate.path.button:updateProperties       { enable=argv.pathOn }
        animate.path.editor:updateProperties       { enable=argv.pathOn }
    else
        animate.path.button:updateProperties       { enable=false }
        animate.path.editor:updateProperties       { enable=false }
    end

    animate.imageFormat.combo:updateProperties     { enable=false }
    animate.videoFormat.combo:updateProperties     { enable=false }
    animate.forceSequence.check:updateProperties   { enable=false }
    animate.startSequence.numeric:updateProperties { enable=false }
    animate.samples.slider:updateProperties        { enable=false }
    animate.frameRate.slider:updateProperties      { enable=false }
    animate.shutterTime.slider:updateProperties    { enable=false }
    animate.startFrame.slider:updateProperties     { enable=false }
    animate.endFrame.slider:updateProperties       { enable=false }
    animate.step.slider:updateProperties           { enable=false }
    animate.render.button:updateProperties         { enable=false }
    animate.stop.button:updateProperties           { enable=false }
    animate.ffmpeg.button:updateProperties         { enable=false }
end


--////////////////////////////////////////////////////////////////////////////
--//
--//                Create video from png or exr files
--//
--////////////////////////////////////////////////////////////////////////////

function animate:createVideo()
    -- First check if ffmpeg exists in script directory. If not, notify where
    -- to get it. Check must be make for .exe and unix style before deciding.
    local win_ffmpeg   = Render.scriptsPath .. "/ffmpeg.exe"
    local unix_ffmpeg  = Render.scriptsPath .. "/ffmpeg"
    win_ffmpeg = win_ffmpeg:gsub("\\", "/")
    unix_ffmpeg = win_ffmpeg:gsub("\\", "/")

    local file, status = io.open(win_ffmpeg, "r")
    if status then
        file, status = io.open(unix_ffmpeg, "r")
        if status then
            local msg1 = "Unable to find ffmpeg in your scripts directory:\n\n"
            msg1 = msg1 .. Render.scriptsPath
            msg2 = "Download and install ffmpeg to the scripts directory.\n\n"
            msg2 = msg2 .. "   Visit https://www.ffmpeg.org"
            dialog.info(msg1, msg2)
            return false
        end
        file:close()
    end

    if animate.path.editor.text == "" then 
        return false
    end
    local path      = animate.path.editor.text
    local fps       = animate.frameRate.slider.value
    local ffmpeg    = [["]]..Render.scriptsPath.."\\ffmpeg"..[["]] .. " -i \""
    local images    = path .."\"".."%06d"..animate:get('imageExtension')
    local codec     = " -c:v libx264 -r "
    local pixFormat = " -pix_fmt yuv420p "
    local videoFile = "\"".. path ..animate:get('videoExtension').."\""
    local script    = (ffmpeg..images..codec..fps..pixFormat..videoFile):gsub("\\",  "/")
    os.execute([["]]..script..[["]])
    return true
end

--------------------------------------------------------------------------------
-- The animate output path and prefix needs to be rebuilt when target changes
-- or other events effecting the name are made.
--------------------------------------------------------------------------------
function animate:rebuildOutputName()
    if animate.path.editor.text == "" then
        names:buildNames(false)
        return false
    end
    local name = names:buildNames(true)
    local temp = string.format("%s/%s", animate.path.path, name)
    animate.path.editor:updateProperties { text=temp:gsub("\\",  "/") }
    return true
end

--------------------------------------------------------------------------------
-- Animate  gui callbacks
--------------------------------------------------------------------------------
function animate:setupCallbacks()

    ----------------------------------------------------------------------
    animate.stop.button.callback   = function() Render:cancel()       end
    animate.ffmpeg.button.callback = function() animate:createVideo() end

    ----------------------------------------------------------------------
    local function getPath()
        Render:cancel()
        -- Choose an output file
        local ret = octane.gui.showDialog
        {
            type  = octane.gui.dialogType.FILE_DIALOG,
            title = "Choose the render output path",
            path  = Render.projectPath,
            save  = false,
            browseDirectory = true
        }
        -- if a path is chosen
        if ret.result ~= "" then
            animate.path.path = ret.result:gsub("\\",  "/")
            local name = names:buildNames(true)
            local temp = string.format("%s/%s", ret.result, name)
            animate.path.editor:updateProperties { text=temp:gsub("\\",  "/") }
        else
            animate.path.editor:updateProperties { text="" }
            animate.path.path = ""
        end

        gui:enable()
        animate.imageFormat.ignoreCallback = true
        animate.videoFormat.ignoreCallback = true

    end

    ----------------------------------------------------------------------
    local function verifyPath()
        local function showInvalidPath(path)
            local msg = "The output directory:\n\n    ".. path .."\n\n"
            msg = msg .. "is invalid. Please try again."
            dialog.critical(msg, false)
        end

        -- Determine if this is Windows or Unix style path
        local isWindows = false
        if Render.projectPath:sub(2,2) == ':' then
            isWindows = true
        end

        -- Set to forward slash 
        animate.path.path = animate.path.path:gsub("\\",  "/")

        local pos = animate.path.path:len()
        -- Make sure path ends with /
        if  animate.path.path:sub(pos,pos) ~= '/' then
            animate.path.path = animate.path.path .. '/'
        end

        -- Make sure beginning of path is absolute for Windows and Unix
        if animate.path.path:sub(1,1) ~= '/' then
             if animate.path.path:sub(2,2) ~= ':' then
                 showInvalidPath(animate.path.path)
                 return false
             elseif animate.path.path:sub(3,3) ~= '/' then
                 showInvalidPath(animate.path.path)
                 return false
             end
        elseif isWindows then
            showInvalidPath(animate.path.path)
            return false
        end

        animate.path.verify = false
        animate.path.button.text = "Output..."
        animate.path.button.tooltip = animate.path.saveTip
        animate.videoFormat.ignoreCallback = true
        animate.imageFormat.ignoreCallback = true
        gui:enable {all=true}

        local path,filename,ext = names.validate:parsePath(animate.path.path)
        local name = names:buildNames(true)
        local temp = ""
        if path ~= nil then
            animate.path.path = path
            if filename ~= nil then
                temp = string.format("%s%s%s", path, filename, name)
            else
                temp = string.format("%s%s", path, name)
            end
        elseif filename ~= nil then
            temp = string.format("%s%s", filename, name)
        end
        animate.path.editor:updateProperties { text=temp:gsub("\\",  "/") }
        local x = animate.path.path:sub(1,string.len(animate.path.path)-1)
        animate.path.path = x
        animate.path.button.callback = getPath
    end

    ----------------------------------------------------------------------
    animate.path.button.callback = getPath

    ----------------------------------------------------------------------
    animate.path.editor.callback = function()
        if animate.path.editor.text ~= "" then
            if animate.path.verify == false then
                local name = names:buildNames(true)
                local stat = string.find(animate.path.editor.text, name)
                if stat then
                    animate.path.editor.text =  animate.path.editor.text:gsub(name,"")
                elseif animate.path.path ~= "" then
                    animate.path.editor.text = animate.path.path
                end
                animate.path.verify = true
                animate.path.button.text = "Accept"
                animate.path.saveTip = animate.path.button.tooltip
                if widget.hideToolTips then
                    animate.path.button.tooltip = ""
                else
                    animate.path.button.tooltip = "Press to accept path"
                end
                animate.path.button.callback = verifyPath
                gui:disable {all=true, exceptAnimate=true}
            end
            animate.path.path = animate.path.editor.text
        elseif animate.path.verify == true then
            animate.path.verify = false
            animate.path.button.text = "Output..."
            animate.path.button.tooltip = animate.path.saveTip
            animate.path.saveTip = ""
            gui:enable {animate_pathOn=true}
            animate.path.path = ""
            animate.path.button.callback = getPath
        end
    end

    ---------------------------------------------
    animate.render.button.callback = function()
         -- Test that the completed output directory exists before starting render,
         -- otherwise the user needs to wait for 1st render to finish before discovering
         -- the file has nowhere to go.
         if animate.path.path ~= "" then
             local name = names:buildNames(true)
             local temp = string.format("%s/%s.png", animate.path.path, name)
             local text = temp:gsub("\\",  "/")
             local path = names.validate:parsePath(text)
             if octane.file.exists(path) == false then
                local msg = "The following output directory does not exist:\n\n    ".. path .."\n\n"
                msg = msg .. "Make sure directory exists before starting render."
                dialog.critical(msg, false)
                return false
             end  
         end  

        if animate.startFrame.slider.value > animate.endFrame.slider.value then
            dialog.critical("Start frame may not be greater than End frame", false)
        else
            renderTarget.combo:updateProperties  { enable=false}
            gui:disable()
            animate.stop.button:updateProperties { enable=true }

            -- Start the actual rendering.
            local saveImage = true
            Render:start(animate.path.editor.text,
                        animate.startFrame.slider.value,
                        animate.endFrame.slider.value,
                        animate.step.slider.value, saveImage)

            if Render.is_dead == false then
                gui:enable()
                renderTarget.ignoreCallback = true
                renderTarget.combo:updateProperties { enable=true}
            end
        end
    end

    ---------------------------------------------
    local function setProjectChanged()
        project.changed = true 
    end

    animate.samples.slider.callback = function()
        setProjectChanged()
        animate:rebuildOutputName()
    end

    animate.frameRate.slider.callback      = setProjectChanged
    animate.shutterTime.slider.callback    = setProjectChanged
    animate.startFrame.slider.callback     = setProjectChanged
    animate.endFrame.slider.callback       = setProjectChanged
    animate.step.slider.callback           = setProjectChanged
    animate.forceSequence.check.callback   = setProjectChanged
    animate.startSequence.numeric.callback = setProjectChanged

    ---------------------------------------------
    animate.imageFormat.ignoreCallback = true

    animate.imageFormat.combo.callback = function()
        if animate.path.verify == true then
            return
        end
        if animate.path.editor.text ~= "" then
            animate:rebuildOutputName()
        end
        if animate.imageFormat.ignoreCallback then
            animate.imageFormat.ignoreCallback = false
        else
            project.changed = true 
        end
    end

    ---------------------------------------------
    animate.videoFormat.ignoreCallback = true

    animate.videoFormat.combo.callback  = function()
        if animate.path.verify == true then
            return
        end
        if animate.path.editor.text ~= "" then
            animate:rebuildOutputName()
        end
        if animate.videoFormat.ignoreCallback then
            animate.videoFormat.ignoreCallback = false
        else
            project.changed = true 
        end
    end

end
