
-- :::::::::::::::::::::::::::::
-- ::                         ::
-- ::      File Names Tab     ::
-- ::                         ::
-- :::::::::::::::::::::::::::::................................................

names = widget:create()
names:new('label','info')
          {"Select components for image output names. Except for 'Text', "
           .. "other values change based on render target, etc...",
           height=40, width=350}
names:new('editor','path') {text="", width=500}
--------------------------------------------------------------------------------

local choiceList =
            { items = {
                   "Text",
                   "Render target",
                   "Resolution",
                   "Samples/pp",
                   "Kernel",
                   "MultiPass"
                   }
             }
-----------------------------------
local staticList =
            { items = {
                   "Sequence",
                   "Extension"
                   }
             }
-----------------------------------

names.choice = {}
for i=1,8 do
    names.choice[i] = names:new('combo') { items=choiceList.items, height=20 }
                       :append('editor') { width=230, height=20 }
end
for i=9,10 do
    names.choice[i] = names:new('combo') { items=staticList.items, height=20 }
                       :append('editor') { width=230, height=20 }
end

--------------------------------------------------------------------------------------


--------------------------------------------------------------------------------------
-- Screen groups and layout for the Settings tab
--------------------------------------------------------------------------------------
do
    local children =
    {
        {names.info.label},
    }
    local g1 = names:new('group') { children, padding={0,2} }

    ---------------------------------------------------------------------
    children = {}
    for i=1, #names.choice do
        table.insert(children, {names.choice[i].combo, names.choice[i].editor })
    end
    local g2 = names:new('group') { children, border=false, padding={5,2} }

    ---------------------------------------------------------------------
    children =
    {
        {g1.group},
        {g2.group},
    }
    local g3 = names:new('group') { children, padding={60,5},x=6, center=true }

    ---------------------------------------------------------------------
    children =
    {
        {names.path.editor},
    }
    local g4 = names:new('group') { children, border=true, center=true,
                              text=" Preview file name sample ", padding={10} }

    children =
    {
        {g3.group},
        {g4.group},
    }
    names:append('group') { children, x=16, padding={0,2}, center=true }
end -----------------------------------=---------------------------------

-------------------------------------------------------------------------
function names:enable()
    self.group:updateProperties                { enable=true } 
    for i=1, #self.choice-2 do
        self.choice[i].combo:updateProperties  { enable=true }
        if self.choice[i].combo.selectedIx == 1 then
            self.choice[i].editor:updateProperties { enable=true }
        else
            self.choice[i].editor:updateProperties { enable=false }
        end
    end
    self.choice[9].combo:updateProperties   { selectedIx=1,  enable=false }
    self.choice[9].editor:updateProperties  { text='000001', enable=false }
    self.choice[10].combo:updateProperties  { selectedIx=2,  enable=false }
    self.choice[10].editor:updateProperties { enable=false }
end

--------------------------------------------------------------------
function names:disable()
    self.group:updateProperties                { enable=false } 
    for i=1, #self.choice do
        self.choice[i].combo:updateProperties  { enable=false }
        self.choice[i].editor:updateProperties { enable=false }
    end
    self.choice[1].editor.enable = false
end

--------------------------------------------------------------------
function names:buildNames(final)
    local path = ""
    for i=1, #self.choice-2 do
        if self.choice[i].combo.selectedIx == 2 then
            if Render.targetNode ~= nil then
                self.choice[i].editor.text =
                    Render.targetNode.name .. "_"
            else
                self.choice[i].editor.text = "Target_"
            end
        elseif self.choice[i].combo.selectedIx == 3 then
            Render:getResolution()
            if Render.resolution.x ~= 0 then
                self.choice[i].editor.text =
                    Render.resolution.x .. "x" ..
                    Render.resolution.y .. "_"
            else
                self.choice[i].editor.text = "1024x512_"
            end
        elseif self.choice[i].combo.selectedIx == 4 then
            self.choice[i].editor.text = animate.samples.slider.value.."spp_"
        elseif self.choice[i].combo.selectedIx == 5 then
            Render:getKernelType()
            if Render.kernel.kType ~= "" then
                self.choice[i].editor.text = Render.kernel.kType .. "_"
            else
                self.choice[i].editor.text = "INFO_"
            end
        elseif self.choice[i].combo.selectedIx == 6 then
            Render:getInfoChannelType()
            if Render.infoChannel.icType ~= "" then
                self.choice[i].editor.text = Render.infoChannel.icType .. "_"
            else
                if final == false then
                    self.choice[i].editor.text = "xx_"
                else
                    self.choice[i].editor.text = ""
                end
            end
        end
        path = path .. self.choice[i].editor.text
    end

    local ext = animate:get('imageExtension')
    self.choice[10].editor.text = ext
    if path ~= "" then
        self.path.editor.text = ".../" .. path .. "000001" .. ext
    else
        self.path.editor.text = path
    end
    return path
end

--------------------------------------------------------------------
function names:saveDefaults(file)
    local endL = "\n"
    self:buildNames(false)
    for i=1, #self.choice do
        file:write("names.choice["..i.."].combo.selectedIx=" .. self.choice[i].combo.selectedIx ..endL)
        file:write("names.choice["..i.."].editor.text='" .. self.choice[i].editor.text .. "'" .. endL)
    end 
end

--------------------------------------------------------------------------------
-- File Name gui callbacks /////////////////////////////////////////////////////
--------------------------------------------------------------------------------
function names:setupCallbacks()

    ---------------------------------------------------------------
    -- Initialize choice list callbacks
    ---------------------------------------------------------------
    for i=1, #self.choice-2 do
        -----------------------------------------------
        self.choice[i].combo.callback = function()
            if self.choice[i].combo.selectedIx == 1 then
                self.choice[i].editor:updateProperties { enable=true }
            else
                self.choice[i].editor:updateProperties { enable=false, text="" }
            end
            self:buildNames(false)
        end

        -----------------------------------------------
        self.choice[i].editor.callback = function()
            self:buildNames(false)
        end

        -----------------------------------------------

    end

    return true
end


----------------------------------------------------------------------------------
-- General validator for path and file names. Determines whether proper extensions
-- are included for paths, whether names are blank, etc..
----------------------------------------------------------------------------------
names.validate = {

    parsePath = function(self, pathName)
        local path, name, ext = 
            string.match(pathName, "(.-)([^\\/]-%.?([^%.\\/]*))$")
        return path, name, ext
    end,

    ----------------------------------------------------
    pathForOpen = function(self, pathName, useExtension)

        local path,filename,ext = self:parsePath(pathName)

        if filename == "" then
            dialog.critical("File name is missing.", false)
            return false
        end
        -- If the filename and extension are equal, there is just a file name
        local fullname = ""
        if filename == ext then
            fullname = pathName .. "." .. useExtension
            filename = filename .. "." .. useExtension
        elseif ext ~= useExtension and ext ~= filename then
            local msg = "Unable to load file.\n\n"
                        .. filename .. " must have extension: ."..useExtension
            dialog.critical(msg, false)
            return false
        else
            fullname = pathName
        end
        return true, fullname, filename, ext
    end,

    ----------------------------------------------------
    pathForSave = function(self, pathName, useExtension)

        local path,filename,ext = self:parsePath(pathName)

        if filename == "" then
            dialog.critical("File name is missing.", false)
            return false
        end
        -- If the filename and extension are equal, there is just a file name
        -- If there was an extension, the save dialog already has check if
        -- file existed. ext is blanked out so calling routine won't check
        -- again in case of assuming the correct extension.
        local fullname = ""
        if filename == ext then
            fullname = pathName .. "." .. useExtension
            filename = filename .. "." .. useExtension
            ext = ""
        elseif ext ~= useExtension and ext ~= filename then
            local msg = "Unable to save file.\n\n"
                        .. filename .. " must have extension: ."..useExtension
            dialog.critical(msg, false)
            return false
        else
            fullname = pathName
        end
        return true, fullname, filename, ext
    end
}
