
-- :::::::::::::::::::::::::::::
-- ::                         ::
-- ::      Settings Tab       ::
-- ::                         ::
-- :::::::::::::::::::::::::::::................................................

settings = widget:create()
settings:new('button', 'saveDefaults')    {"Save defaults", width=120, height=40, tooltip="Save defaults to Orc_defaults"}
settings:new('numeric','keyframeMax')    {{"Maximum number of frames", width=190}, value=500, min=10, max=2000, step=100}
settings:new('numeric','keyframeValue')  {{"Default number of frames", width=190}, step=10, value=10, min=1, max=settings.keyframeMax.numeric.value}
settings:new('numeric','keyframeLimit')  {{"Maximum keyframe screen items",width=190}, value=7, min=2, max=30,  step=1}
settings:new('numeric','keyframeSamples'){{"Maximum keyframe samples/px",  width=190}, value=500, min=1, max=256000, step=1}
settings:new('numeric','shutterTime')    {{"Shutter Time",}, value=0.001, min=0, max=2, step=0.001, log=true}
settings:new('numeric','frameRate')      {{"Frame Rate"}, value=25, min=10, max=120}
settings:new('check', 'hideToolTips')    {text="Hide button tool tips", width=200}

----------------------------------------------------------------------------------
-- If the Orc_Defaults file exists it will be loaded, otherwise the regular
-- defaults will kick in. Once the user saves the defaults, they will be
-- available when first running this routine which has to take place before the
-- keyframe screen items are built.
----------------------------------------------------------------------------------
local function loadDefaults()

    local scriptsPath = octane.file.getSpecialDirectories()["userScriptDirectory"]
    local orcPath = scriptsPath .. "/Orc_bin/"
    orcPath = orcPath:gsub("\\", "/")
    local name = orcPath.."Orc_Defaults"

    -----------------------------------------------------------
    -- This block can be used for retrieving the version number
    -- in case action needs to be taken for handling data
    -- of a particular Orc version in a particular way.
    -----------------------------------------------------------
    --[[
    local file, status = io.open(name, "r")
    if status == nil then
        local versionStr = file:read("*l")
        version = versionStr:gsub("OrcDefVersion=","")
        if version ~= versionStr then
            print(version)
        end
        file:close()
    end ]]
    -------------------------------------------------------

    local ff, status = loadfile(name)
    if not status then
        ff() 
        if settings.hideToolTips.check.checked == true then
            widget.hideToolTips = true
            settings.saveDefaults.button:updateProperties {tooltip = " "}
        end
    end
end

----------------------
    loadDefaults()  --
----------------------

--------------------------------------------------------------------------------------
-- Screen groups and layout for the Settings tab
--------------------------------------------------------------------------------------
do 
    local children =
    {
        { settings.shutterTime.label, settings.shutterTime.numeric },
        { settings.frameRate.label,   settings.frameRate.numeric },
    }
    local g1 = settings:new('group') { children, padding={0,2} }

    ---------------------------------------------------------------------
    children =
    {
        { settings.hideToolTips.check }
    }
    local g2 = settings:new('group') { children, padding={0,2} }

    ---------------------------------------------------------------------
    children =
    {
        { settings.keyframeValue.label,   settings.keyframeValue.numeric },
        { settings.keyframeMax.label,     settings.keyframeMax.numeric },
        { settings.keyframeLimit.label,   settings.keyframeLimit.numeric },
        { settings.keyframeSamples.label, settings.keyframeSamples.numeric }
    }
    local g3 = settings:new('group') { children, text="  Keyframe  ",
                                       border=true, padding={5,2}, inset={34,0} }

    ---------------------------------------------------------------------
    children =
    {
        { settings.saveDefaults.button }
    }
    local g4 = settings:new('group') { children, padding={0,15 } }

    ---------------------------------------------------------------------
    children =
    {
        { g1.group },
        { g2.group },
        { g3.group },
        { g4.group }
    }
    local globals = settings:new('group') { children, 
                                            padding={20,5}, center=true } 

    ---------------------------------------------------------------------
    children =
    {
        { globals.group }
    }
    settings:append('group') { children, padding={0,20}, inset={70,0}, center=true }
end ---------------------------------------------------------------------------

--------------------------------------------------------------------------------
function settings:enable()
 
    local enabler = cameraFile.editor.text ~= ""

    self.saveDefaults.button:updateProperties { enable=enabler }
end

--------------------------------------------------------------------
function settings:disable()

    self.saveDefaults.button:updateProperties { enable=false }
end

--------------------------------------------------------------------------------
local function saveDefaults()

    local endL = "\n"
    local orcPath = Render.scriptsPath .. "/Orc_bin/"
    orcPath = orcPath:gsub("\\", "/")
    local name = orcPath.."Orc_Defaults"

    local file, status = io.open(name, "wb")
    if status then
        local msg = "Unable to create " .. name ..
                    "\nMake sure the directory:" ..orcPath.. " exists.\n"
        msg = msg:gsub("\\", "/")
        msg = msg .. "Status:" .. status
        dialog.critical(msg, false)
        return false
    end
    file:write("OrcDefVersion="                          .. VERSION..endL)
    file:write("settings.shutterTime.numeric.value="     .. settings.shutterTime.numeric.value ..endL)
    file:write("settings.frameRate.numeric.value="       .. settings.frameRate.numeric.value ..endL)
    file:write("settings.hideToolTips.check.checked="    .. tostring(settings.hideToolTips.check.checked) ..endL)
    file:write("settings.keyframeValue.numeric.value="   .. settings.keyframeValue.numeric.value ..endL)
    file:write("settings.keyframeMax.numeric.value="     .. settings.keyframeMax.numeric.value ..endL)
    file:write("settings.keyframeLimit.numeric.value="   .. settings.keyframeLimit.numeric.value ..endL)
    file:write("settings.keyframeSamples.numeric.value=" .. settings.keyframeSamples.numeric.value ..endL)

    names:saveDefaults(file)  -- Save default settings for file name layout

    file:flush()
    file:close()

    dialog.info("Default settings saved.",
                "These changes will only take effect\nwhen the script is restarted.\n")
    return true
end

--------------------------------------------------------------------------------
-- Settings gui callbacks
--------------------------------------------------------------------------------
function settings:setupCallbacks()

    self.saveDefaults.button.callback = function()
        saveDefaults()
    end

    return true
end

