
-- :::::::::::::::::::::::::::::
-- ::                         ::
-- ::      Solar Tab          ::
-- ::                         ::
-- :::::::::::::::::::::::::::::................................................

solar = widget:create()
solar:new('check', 'enableSun')  { label={"Enable sun movement", x=-12, width=160}, width=18, x=2}
solar:new('slider','startTime')  {{"Start Time", width=60}, width=390, value=12, min=0, max=24, step=0.25 }
solar:new('slider','endTime')    {{"End Time",   width=60}, width=390, value=12, min=0, max=24, step=0.25 }
solar:new('slider','preview')    {{"Preview",    width=60}, width=390, value=12, min=0, max=24, step=0.25 }
solar:new('label', 'startHMS')   {{"12:00:00",   width=70}}
solar:new('label', 'endHMS')     {{"12:00:00",   width=70}}
solar:new('label', 'previewHMS') {{"12:00:00",   width=70}}

solar.overrideTime = false -- Used when previewing time of day

--------------------------------------------------------------------------------------
-- Screen groups and layout for the Solar tab
--------------------------------------------------------------------------------------
do
    local children =
    { 
        { solar.enableSun.check, solar.enableSun.label },
    }
    local g1 = solar:new('group') { children, padding={10,0} }

    ---------------------------------------------------------------------
    children =
    { 
        { solar.preview.label, solar.previewHMS.label, solar.preview.slider }
    }
    local g2 = solar:new('group') { children, padding={0,2} }

    ---------------------------------------------------------------------
    children =
    { 
        { solar.startTime.label, solar.startHMS.label, solar.startTime.slider },
        { solar.endTime.label,   solar.endHMS.label,   solar.endTime.slider },
    }
    local g3 = solar:new('group') { children, padding={0,2} }

    ---------------------------------------------------------------------
    children =
    {
        {g1.group},
        {g2.group},
        {g3.group},
    }
    solar:append('group') { children, border=true, padding={5,10}, x=10,y=10 }
end  ---------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Solar functions /////////////////////////////////////////////////////////////
--------------------------------------------------------------------------------
function solar:enable(...)

    local argv = {...} if (#argv > 0) then argv = argv[1] end

    if argv.check ~= nil then
       self.enableSun.check:updateProperties   { enable=argv.check }
       self.enableSun.label:updateProperties   { enable=argv.check }
    end

    if self.enableSun.check.checked then
        self.startTime.slider:updateProperties { enable=true }
        self.endTime.slider:updateProperties   { enable=true }
        self.preview.slider:updateProperties   { enable=true }
    else
        Render.envNode = nil
        Render.sunDirNode = nil
    end
end

-----------------------------------------------------------------
function solar:disable(...)

    local argv = {...} if (#argv > 0) then argv = argv[1] end

    if argv.setCheck ~= nil then
        self.enableSun.check:updateProperties { enable=true }
        self.enableSun.label:updateProperties { enable=true }
    else
        self.enableSun.check:updateProperties { enable=false }
        self.enableSun.label:updateProperties { enable=false }
    end
    self.startTime.slider:updateProperties    { enable=false }
    self.endTime.slider:updateProperties      { enable=false }
    self.preview.slider:updateProperties      { enable=false }
end

--------------------------------------------------------------------------------
-- Solar gui callbacks
--------------------------------------------------------------------------------
function solar:setupCallbacks()

    -------------------------------------------------
    self.enableSun.check.callback = function()

        if self.enableSun.check.checked then
            Render.envNode = Render.targetNode:getInputNode(octane.P_ENVIRONMENT)
            if not Render.envNode then
                local msg = "A Daylight environment node must be attached to the render target "
                msg = msg.."in order to use sun movement within the animator script."
                msg = msg.."Sun movement will be ignored."
                dialog.warning("Environment node is missing.",msg)
            elseif Render.envNode:getProperties().type == octane.NT_ENV_DAYLIGHT then
                Render.sunDirNode = Render.envNode:getInputNode(octane.P_SUN_DIR)
                if not Render.sunDirNode or Render.sunDirNode:getProperties().type ~= octane.NT_SUN_DIRECTION then
                    local msg = "Daylight environment has no time or location associated with it. "
                    msg = msg.."Sun movement will be ignored."
                    dialog.warning("Sun direction is missing.",msg)
                else
                    gui:enable()
                    return true
                end
            end
        end
        self.enableSun.check.checked = false
        Render.envNode = nil
        Render.sunDirNode = nil
        self:disable {setCheck=true}
        return false
    end

    -------------------------------------------------
    local function formatTime(timeValue)
        local h = math.floor(timeValue)
        local t = timeValue - h
        local m = math.floor(t * 60)
        local s = math.floor((t - m/60) * 3600)
        return string.format("%02d:%02d:%02d",h,m,s)
    end

    -------------------------------------------------
    solar.startTime.slider.callback = function()
        solar.startHMS.label.text = formatTime(solar.startTime.slider.value)
        project.changed = true
    end

    -------------------------------------------------
    solar.endTime.slider.callback = function()
        solar.endHMS.label.text = formatTime(solar.endTime.slider.value)
        project.changed = true
    end

    -------------------------------------------------
    solar.preview.slider.callback = function()
        solar.previewHMS.label.text = formatTime(solar.preview.slider.value)
        if Render.sunDirNode ~= nil then
            Render:cancel()
            Render.sunDirNode:setPinValue(octane.P_HOUR, solar.preview.slider.value)
            solar.overrideTime = true
            keyframe:renderFrame()
            solar.overrideTime = false
        end
    end

    return true
end

